import torch
import torch.nn as nn
import torch.optim as optim
from sklearn.preprocessing import MinMaxScaler
from torch.utils.data import DataLoader, TensorDataset
import pandas as pd
from sklearn.model_selection import train_test_split
import numpy as np
from sklearn.metrics import mean_squared_error, mean_absolute_error
import numpy as np

data = pd.read_excel('data/2023+.xlsx')
data.fillna(data.mean(), inplace=True)
validation_set = data.iloc[-30*24:]
data = data.iloc[:-30*24]

# 特征和标签
features = data[['hour', 'AQI', 'SO2', 'NO2', 'O3', 'CO']]
labels = data['PM2.5']

# 训练集和测试集切分为8:2
X_train, X_test, y_train, y_test = train_test_split(features, labels, test_size=0.2, random_state=42)


# 保存数据
X_train.to_csv('data/X_train.csv', index=False)
X_test.to_csv('data/X_test.csv', index=False)
y_train.to_csv('data/y_train.csv', index=False)
y_test.to_csv('data/y_test.csv', index=False)
validation_set.to_csv('validation_set.csv', index=False)


# 标准化数据
scaler = MinMaxScaler()
X_train_scaled = scaler.fit_transform(X_train)
X_test_scaled = scaler.transform(X_test)
X_val_scaled = scaler.transform(validation_set[['hour', 'AQI', 'SO2', 'NO2', 'O3', 'CO']])
y_val = validation_set['PM2.5'].values

# 转换为PyTorch张量
X_train_tensors = torch.tensor(X_train_scaled, dtype=torch.float32)
y_train_tensors = torch.tensor(y_train.values, dtype=torch.float32).view(-1, 1)
X_test_tensors = torch.tensor(X_test_scaled, dtype=torch.float32)
y_test_tensors = torch.tensor(y_test.values, dtype=torch.float32).view(-1, 1)
X_val_tensors = torch.tensor(X_val_scaled, dtype=torch.float32)
y_val_tensors = torch.tensor(y_val, dtype=torch.float32).view(-1, 1)

# 创建数据加载器
train_dataset = TensorDataset(X_train_tensors, y_train_tensors)
test_dataset = TensorDataset(X_test_tensors, y_test_tensors)
val_dataset = TensorDataset(X_val_tensors, y_val_tensors)

train_loader = DataLoader(train_dataset, batch_size=32, shuffle=True)
test_loader = DataLoader(test_dataset, batch_size=32, shuffle=False)
val_loader = DataLoader(val_dataset, batch_size=32, shuffle=False)


class LSTMModel(nn.Module):
    def __init__(self, input_size, hidden_size, num_layers, output_size):
        super(LSTMModel, self).__init__()
        self.hidden_size = hidden_size
        self.num_layers = num_layers
        self.lstm = nn.LSTM(input_size, hidden_size, num_layers, batch_first=True)
        self.fc = nn.Linear(hidden_size, output_size)

    def forward(self, x):
        h0 = torch.zeros(self.num_layers, x.size(0), self.hidden_size)
        c0 = torch.zeros(self.num_layers, x.size(0), self.hidden_size)
        out, _ = self.lstm(x, (h0, c0))
        out = self.fc(out[:, -1, :])
        return out

# 超参数
input_size = X_train.shape[1]
hidden_size = 64
num_layers = 2
output_size = 1
num_epochs = 200
learning_rate = 0.001

# 实例化模型、损失函数和优化器
model = LSTMModel(input_size, hidden_size, num_layers, output_size)
criterion = nn.MSELoss()
optimizer = optim.Adam(model.parameters(), lr=learning_rate)

# 训练模型
for epoch in range(num_epochs):
    model.train()
    for X_batch, y_batch in train_loader:
        X_batch = X_batch.unsqueeze(1)
        outputs = model(X_batch)
        loss = criterion(outputs, y_batch)
        optimizer.zero_grad()
        loss.backward()
        optimizer.step()

    if (epoch + 1) % 10 == 0:
        print(f'Epoch [{epoch + 1}/{num_epochs}], Loss: {loss.item():.4f}')
# 保存模型
torch.save(model.state_dict(), 'model/lstm_model.pth')

from sklearn.metrics import mean_squared_error, mean_absolute_error
import numpy as np
import openpyxl

model.eval()
with torch.no_grad():
    X_val_tensors = X_val_tensors.unsqueeze(1)
    y_val_pred = model(X_val_tensors).numpy()

# 计算误差指标
mse = mean_squared_error(y_val, y_val_pred)
rmse = np.sqrt(mse)
mae = mean_absolute_error(y_val, y_val_pred)

print(f'MSE: {mse:.4f}')
print(f'RMSE: {rmse:.4f}')
print(f'MAE: {mae:.4f}')

# 保存误差指标到txt文件
with open('output/error_metrics.txt', 'w') as f:
    f.write(f'MSE: {mse:.4f}\n')
    f.write(f'RMSE: {rmse:.4f}\n')
    f.write(f'MAE: {mae:.4f}\n')

# 保存预测结果到Excel文件
results_df = pd.DataFrame({'真实值': y_val, '预测值': y_val_pred.flatten()})
results_df.to_excel('output/预测结果.xlsx', index=False)
