import torch
import torch.nn as nn
import torch.optim as optim
import torchvision
import torchvision.datasets as datasets
import torchvision.transforms as transforms
import matplotlib.pyplot as plt 
from torch.utils.data import DataLoader
from torch.utils.tensorboard import SummaryWriter
 #定义超参数
EPOCH = 10
BATCH_SIZE = 64
LR = 0.001
TIME_STEP = 28
INPUT_SIZE = 28
#数据加载
train_data = datasets.MNIST (root='./mnist',train=True,transform=transforms.ToTensor(),download=True )
test_data = datasets.MNIST(root='./mnist',train=False,
                              transform=transforms.ToTensor(),download=True )
                            
test_x = test_data.test_data.type(torch.FloatTensor)[:2000]/255#归一化
test_y = test_data.test_labels.numpy()[:2000]
#数据分批
train_loader = DataLoader(dataset=train_data,batch_size=BATCH_SIZE,shuffle=True)
test_loader = DataLoader(dataset=test_data,batch_size=BATCH_SIZE,shuffle=True)
#定义网络
class RNN(nn.Module):
    def __init__(self):
        super(RNN,self).__init__()
        #输入是输入特征的维度input_size=28，中间隐藏层的神经元个数hidden_size指定为64，网络的层数设置为1，
        #全连接层输出的大小为10
        input_size,hidden_size,output_size = INPUT_SIZE,64,10
        num_layers = 1
        self.rnn = nn.RNN(input_size,hidden_size,num_layers,batch_first=True)
        self.out = nn.Linear(hidden_size,output_size)
    def forward(self,x):
        r_out,_ = self.rnn(x)
        out = self.out(r_out[:,-1,:])
        return out
#设置使用GPU
device = torch.device('cuda:0' if torch.cuda.is_available() else 'cpu')
#使用定义好的RNN
model = RNN()
#模型和输入数据都需要to device
mode  = model.to(device)
#定义损失函数,分类问题使用交叉信息熵
criterion = nn.CrossEntropyLoss()
optimizer = torch.optim.Adam(model.parameters(),lr= LR)
#开始训练
writer = SummaryWriter('cifar-10')
for epoch in range(EPOCH):
    for step,data in enumerate(train_loader):
        #取出数据及标签
        inputs,labels = data
        inputs = inputs.view(-1,28,28)
        #数据及标签均送入GPU或CPU
        inputs,labels = inputs.to(device),labels.to(device) 
        #前向传播
        outputs = model(inputs)
        #计算损失函数
        loss = criterion(outputs,labels)
        #清空上一轮的梯度
        optimizer.zero_grad()      
        #反向传播
        loss.backward()
        #参数更新
        optimizer.step()
        #测试，计算准确率
        if  step%100 == 0:
            writer.add_scalar("Train/Loss", loss.item(), epoch*len(train_loader)+step)
            test_outputs = model(test_x.cuda())
            pred_y = torch.max(test_outputs,1)[1].data.cpu().numpy()
            accuracy = float((pred_y == test_y).astype(int).sum())/ float(test_y.size)
            print ('Epoch: {}, Step: {}, loss: {}, accuracy: {}'.format(epoch, step, loss.item(), accuracy))
            writer.add_scalar("Accuracy", 100.0*accuracy, epoch*len(train_loader)+step)
# 从测试集选出30个，进行验证
test_x = test_x.cuda()
test_output = model(test_x[:30].view(-1, 28, 28))
pred_y = torch.max(test_output, 1)[1].data.cpu().numpy()
print('预测数字', pred_y)
print('实际数字', test_y[:30])

