import re
import numpy as np
import pandas as pd
from sklearn.model_selection import train_test_split
import torch
from torch import nn, optim
from torch.utils.data import DataLoader, Dataset
from sklearn.metrics import classification_report
import optuna


import nltk
import nltk

# 设置清华大学的NLTK数据源
nltk.set_url('https://mirrors.tuna.tsinghua.edu.cn/nltk_data/')
nltk.u
# 检查是否设置成功
print(nltk.data.show_install_info())

class Args:
    max_vocab_size = 25000  # 限定词表最大规模
    n_labels = 5            # 情感类别个数
    epochs = 5              # 训练总周期
    embedding_dim = 300     # 词向量维度
    hidden_dim = 512        # LSTMCell隐层维度
    n_layers = 3            # LSTM层数
    batch_size = 64         # 批样本量大小
    display_freq = 50       # 输出信息间隔
    lr= 0.01                # 学习率
# 实例化
args = Args()

# text = data.Field()
# label = data.LabelField(dtype = torch.float)




nltk.download('punkt')
# nltk.download('averaged_perceptron_tagger')
nltk.download('stopwords')

from nltk.tokenize import word_tokenize
from nltk.corpus import stopwords


def preprocess_text(text):
    # 这里只进行分词，实际应用中可能还需要去除停用词、词形还原等
    return word_tokenize(text.lower())
stop_words = set(stopwords.words('chinese'))
from nltk.tokenize import word_tokenize
from collections import Counter

def preprocess_text(text):
    # 这里只进行分词，实际应用中可能还需要去除停用词、词形还原等
    word=word_tokenize(text.lower())
    return word



file_path = 'results/train_data.csv'
data = pd.read_csv(file_path)

# 清理文本数据
def clean_text(text):
    text = re.sub(r'[^a-zA-Z0-9\s]', '', text, re.I | re.A)
    text = text.lower()
    text = text.strip()
    return text




# 合并相关文本列进行分析
data['text'] = data['title'].fillna('') + " " + data['company_profile'].fillna('') + " " + data['description'] + " " + \
               data['requirements'].fillna('') + " " + data['benefits'].fillna('')
# processed_texts = [preprocess_text(text) for text in data['text'].values]
# word_counts = Counter(word for text in processed_texts for word in text)
data['text'] = data['text'].apply(clean_text)

from nltk.corpus import stopwords
import nltk
def separate_sentence(text):
    disease_List = nltk.word_tokenize(text)
    #去除停用词
    filtered = [w for w in disease_List if(w not in stopwords.words('my_english'))]
    #进行词性分析，去掉动词、助词等
    Rfiltered =nltk.pos_tag(filtered)
    #以列表的形式进行返回，列表元素以（词，词性）元组的形式存在
    filter_word = [i[0] for i in Rfiltered]
    return " ".join(filter_word)
data['text'] = data['text'].apply(lambda x:separate_sentence(x))

#用xxx条玩玩

sentences = list(data['text'])
labels = list(data['fraudulent'])
max_seq_len = max(data['text'].apply(lambda x: len(x.split())))
PAD = ' <PAD>'  # 未知字，padding符号用来填充长短不一的句子(用啥符号都行，到时在nn.embedding的参数设为padding_idx=word_to_id['<PAD>'])即可

#小于最大长度的补齐
for i in range(len(sentences)):
    sen2list = sentences[i].split()
    sentence_len = len(sen2list)
    if sentence_len<max_seq_len:
        sentences[i] += PAD*(max_seq_len-sentence_len)

num_classes = len(set(labels))  # num_classes=2
word_list = " ".join(sentences).split()
vocab = list(set(word_list))
word2idx = {w: i for i, w in enumerate(vocab)}
vocab_size = len(vocab)



def make_data(sentences, labels):
    inputs = []
    for sen in sentences:
        inputs.append([word2idx[n] for n in sen.split()])

    targets = []
    for out in labels:
        targets.append(out) # To using Torch Softmax Loss function
    return inputs, targets

input_batch, target_batch = make_data(sentences, labels)
# input_batch, target_batch = torch.LongTensor(input_batch), torch.LongTensor(target_batch)


# text.build_vocab(
#     processed_texts,
#     max_size = args.max_vocab_size,
#     vectors = "glove.6B.300d",
#     unk_init = torch.Tensor.normal_
# )


# from torchtext.vocab import build_vocab_from_iterator

# 构建词汇表，这里我们只取最常见的N个词
# vocab = build_vocab_from_iterator(word_counts.keys(), specials=None)
# vocab.set_default_index(vocab["<unk>"])  # 未知词设置为<unk>




# 假设你有一系列文本数据
# texts = ["I love natural language processing", "NLP is fun", "Machine learning is also cool",
#          "But NLP is more interesting"]
# processed_texts = [preprocess_text(text) for text in texts]
# 加载数据集





# 对文本数据进行分词
# from sklearn.feature_extraction.text import CountVectorizer
#
# vectorizer = CountVectorizer()
# X = vectorizer.fit_transform(data['text']).toarray()
# vocab_size = len(vectorizer.vocabulary_)

# 提取目标变量
# y = data['fraudulent'].values

# 将数据分为训练集和验证集
X_train, X_val, y_train, y_val = train_test_split(input_batch, target_batch, test_size=0.2, random_state=42)


# 定义数据集类
class JobDataset(Dataset):
    def __init__(self, texts, labels):
        self.texts = texts
        self.labels = labels

    def __len__(self):
        return len(self.texts)

    def __getitem__(self, idx):
        return torch.tensor(self.texts[idx], dtype=torch.float32), torch.tensor(self.labels[idx], dtype=torch.float32)


# 创建训练和验证数据集
train_dataset = JobDataset(X_train, y_train)
val_dataset = JobDataset(X_val, y_val)

# 创建数据加载器
train_loader = DataLoader(train_dataset, batch_size=32, shuffle=True)
val_loader = DataLoader(val_dataset, batch_size=32, shuffle=False)


# 定义LSTM分类器
class LSTMClassifier(nn.Module):
    def __init__(self, vocab_size, embedding_dim, hidden_dim):
        super(LSTMClassifier, self).__init__()
        self.embedding = nn.Embedding(vocab_size, embedding_dim)
        self.lstm = nn.LSTM(embedding_dim, hidden_dim, batch_first=True)
        self.fc = nn.Linear(hidden_dim, 1)
        self.sigmoid = nn.Sigmoid()

    def forward(self, x):
        x = self.embedding(x)
        x, (h, c) = self.lstm(x)
        x = self.fc(x[:, -1, :])
        x = self.sigmoid(x)
        return x


# 训练模型
def train_model(model, criterion, optimizer, train_loader, val_loader, epochs=10):
    for epoch in range(epochs):
        model.train()
        for texts, labels in train_loader:
            # texts, labels=texts.cuda(), labels.cuda()
            optimizer.zero_grad()
            outputs = model(texts.long())
            loss = criterion(outputs, labels.unsqueeze(1))
            loss.backward()
            optimizer.step()

        model.eval()
        val_loss = 0
        with torch.no_grad():
            for texts, labels in val_loader:
                outputs = model(texts.long())
                loss = criterion(outputs, labels.unsqueeze(1))
                val_loss += loss.item()
        print(f'Epoch {epoch + 1}, Validation Loss: {val_loss / len(val_loader)}')


# 定义Optuna目标函数
def objective(trial):
    embedding_dim = trial.suggest_int('embedding_dim', 2, 8)
    hidden_dim = trial.suggest_int('hidden_dim', 4, 16)
    # embedding_dim = trial.suggest_int('embedding_dim', 32, 128)
    # hidden_dim = trial.suggest_int('hidden_dim', 64, 256)
    lr = trial.suggest_loguniform('lr', 1e-5, 1e-1)

    model = LSTMClassifier(vocab_size=vocab_size, embedding_dim=embedding_dim, hidden_dim=hidden_dim)
    model = model
    criterion = nn.BCELoss()
    optimizer = optim.Adam(model.parameters(), lr=lr)

    train_model(model, criterion, optimizer, train_loader, val_loader, epochs=10)

    val_loss = 0
    model.eval()
    with torch.no_grad():
        for texts, labels in val_loader:
            outputs = model(texts.long())
            loss = criterion(outputs, labels.unsqueeze(1))
            val_loss += loss.item()

    return val_loss / len(val_loader)


# 创建Optuna学习对象
# study = optuna.create_study(direction='minimize')
# study.optimize(objective, n_trials=50)
#
# print('Best trial:')
# trial = study.best_trial
#
# print(f'  Val Loss: {trial.value}')
# print('  Params: ')
# for key, value in trial.params.items():
#     print(f'    {key}: {value}')

# 使用最佳超参数训练最终模型
# best_params = trial.params
final_model = LSTMClassifier(vocab_size=vocab_size, embedding_dim=32,
                             hidden_dim=16)
final_optimizer = optim.Adam(final_model.parameters(), lr=1e-1)

train_model(final_model, nn.BCELoss(), final_optimizer, train_loader, val_loader, epochs=10)

# 保存模型
model_save_path = 'model/model.pt'
torch.save(final_model, model_save_path)
print(f"Model saved to {model_save_path}")


# ********************************下面为预测结果，可以不使用***********************************************


# 评估模型
def evaluate_model(model, data_loader):
    model.eval()
    y_true = []
    y_pred = []
    with torch.no_grad():
        for texts, labels in data_loader:
            outputs = model(texts.long())
            y_true.extend(labels.tolist())
            y_pred.extend(outputs.squeeze().tolist())
    return y_true, y_pred


y_true, y_pred = evaluate_model(final_model, val_loader)
y_pred = [1 if p > 0.5 else 0 for p in y_pred]

print(classification_report(y_true, y_pred))
