import pandas as pd
from sklearn.model_selection import train_test_split
from transformers import LlamaTokenizer, LlamaForSequenceClassification, Trainer, TrainingArguments
from torch.utils.data import DataLoader, Dataset
import torch

# 加载数据集
file_path = 'dataset/temp.csv'
data = pd.read_csv(file_path)

# 映射标签
label_map = {'positive': 1, 'negative': 0}
data['label'] = data['sentiment'].map(label_map)

# 切分数据集
train_texts, val_texts, train_labels, val_labels = train_test_split(data['review'], data['label'], test_size=0.2, random_state=42)

# 加载粉刺起，注意加载方式，或者Auto
tokenizer = LlamaTokenizer.from_pretrained('../Llama2-7B-guanaco-1k')

# 数据转换为列表，启用截断，启用填充
train_encodings = tokenizer(train_texts.tolist(), truncation=True, padding=True)
val_encodings = tokenizer(val_texts.tolist(), truncation=True, padding=True)

# 创建数据类
class SentimentDataset(Dataset):
    def __init__(self, encodings, labels):
        self.encodings = encodings  #文本数据
        self.labels = labels    #标签

    def __getitem__(self, idx):
        item = {key: torch.tensor(val[idx]) for key, val in self.encodings.items()}
        item['labels'] = torch.tensor(self.labels[idx])
        return item

    def __len__(self):
        return len(self.labels)

train_dataset = SentimentDataset(train_encodings, train_labels.tolist())
val_dataset = SentimentDataset(val_encodings, val_labels.tolist())

md=r'C:\models\chatglm3-6b'
# 同样注意加载器 '../Llama2-7B-guanaco-1k'
model = LlamaForSequenceClassification.from_pretrained(md, num_labels=2)

# 定义训练配置
training_args = TrainingArguments(
    output_dir='result',
    num_train_epochs=3,
    per_device_train_batch_size=4,
    per_device_eval_batch_size=4,
    warmup_steps=500,   #学习率跳跃步数
    weight_decay=0.01,  #惩罚权重，每次降低权重的比率，防止过拟合
    logging_dir='./logs',   #日志路径
    logging_steps=10,   #日志步数
)

# 定义训练器
trainer = Trainer(
    model=model,
    args=training_args,
    train_dataset=train_dataset,
    eval_dataset=val_dataset,
)

# 开始训练
trainer.train()
