from transformers import GPT2Tokenizer, GPT2LMHeadModel, Trainer, TrainingArguments
from datasets import load_dataset
from sklearn.model_selection import train_test_split
import pandas as pd
# 加载数据集
file_path = 'dataset/temp.csv'
data = pd.read_csv(file_path)

# 映射标签
label_map = {'positive': 1, 'negative': 0}
data['label'] = data['sentiment'].map(label_map)

# 切分数据集
train_texts, val_texts, train_labels, val_labels = train_test_split(data['review'], data['label'], test_size=0.2, random_state=42)

md=r'C:\models\chatglm3-6b'
from transformers import AutoTokenizer, AutoModel
tokenizer = AutoTokenizer.from_pretrained(r"C:\models\chatglm3-6b", trust_remote_code=True)
model = AutoModel.from_pretrained(r"C:\models\chatglm3-6b", trust_remote_code=True).half().cuda()

train_encodings = tokenizer(train_texts.tolist(), truncation=True, padding=True)
val_encodings = tokenizer(val_texts.tolist(), truncation=True, padding=True)

from torch.utils.data import DataLoader, Dataset
import torch
# 创建数据类
class SentimentDataset(Dataset):
    def __init__(self, encodings, labels):
        self.encodings = encodings  #文本数据
        self.labels = labels    #标签

    def __getitem__(self, idx):
        item = {key: torch.tensor(val[idx]) for key, val in self.encodings.items()}
        item['labels'] = torch.tensor(self.labels[idx])
        return item

    def __len__(self):
        return len(self.labels)

train_dataset = SentimentDataset(train_encodings, train_labels.tolist())
val_dataset = SentimentDataset(val_encodings, val_labels.tolist())

# 假设你有一个数据集，其中包含用于微调的文本序列
# 这里我们使用一个假设的函数来加载和预处理数据
# 注意：你需要根据自己的数据集调整这部分代码


# 定义训练参数
training_args = TrainingArguments(
    output_dir='./fine-tuned-gpt2',  # 输出文件夹
    overwrite_output_dir=True,  # 如果输出文件夹已存在，则覆盖
    num_train_epochs=3,  # 训练轮次
    per_device_train_batch_size=4,  # 每个设备训练批次大小（根据你的硬件资源调整）
    per_device_eval_batch_size=64,  # 对于生成式模型，这个参数可能不是必需的
    warmup_steps=500,  # 预热步数
    weight_decay=0.01,  # 权重衰减
    logging_dir='./logs',  # 日志记录文件夹
    logging_steps=10,
    save_steps=500,  # 每多少步保存一次模型
    save_total_limit=2,  # 保存模型的总数限制
)

# 注意：由于我们没有实际的数据集和Trainer的完整配置，这里我们不会直接实例化Trainer
# 但通常情况下，你会这样做：
# trainer = Trainer(
#     model=model,
#     args=training_args,
#     train_dataset=你的训练数据集编码（需要是一个torch.utils.data.Dataset对象或类似结构）
#     # 注意：对于生成式模型，你可能不需要验证集，或者你可以使用不同的评估方法来监控模型性能
# )

# trainer.train()  # 这将开始训练过程

# 注意：由于上述代码中存在许多假设和省略的部分，你需要根据你的具体需求和数据集来调整它
# 特别是，你需要确保你的数据集被正确地加载、预处理，并且以适合训练的形式提供给Trainer

# 如果你

training_args = TrainingArguments(
    output_dir='result',
    num_train_epochs=3,
    per_device_train_batch_size=4,
    per_device_eval_batch_size=4,
    warmup_steps=500,   #学习率跳跃步数
    weight_decay=0.01,  #惩罚权重，每次降低权重的比率，防止过拟合
    # logging_dir='result',   #日志路径
    logging_steps=10,   #日志步数
)

# 定义训练器
trainer = Trainer(
    model=model,
    args=training_args,
    train_dataset=train_dataset,
    eval_dataset=val_dataset,
)

# 开始训练
trainer.train()