import pandas as pd
from datasets import load_dataset, Dataset
from transformers import AutoTokenizer, LlamaForSequenceClassification, TrainingArguments, Trainer
import torch
import numpy as np
from sklearn.metrics import accuracy_score
# torch.cuda.set_per_process_memory_fraction(0.9)
# 加载数据，注意csv间隔符号
train_data = pd.read_csv('dataset/train.txt', sep='\t', header=None, names=['text1', 'text2', 'label'])
test_data = pd.read_csv('dataset/test.txt', sep='\t', header=None, names=['text1', 'text2', 'label'])
val_data = pd.read_csv('dataset/dev.txt', sep='\t', header=None, names=['text1', 'text2', 'label'])

# 创建 Dataset 对象
train_dataset = Dataset.from_pandas(train_data)
test_dataset = Dataset.from_pandas(test_data)
val_dataset = Dataset.from_pandas(val_data)

# 加载 tokenizer
tokenizer = AutoTokenizer.from_pretrained("../baichuan-LLM", trust_remote_code=True)

# 填充标记用于将不同长度的输入序列扩展到相同的长度，以便批处理
tokenizer.add_special_tokens({'pad_token': '[PAD]'})

# 定义一个函数来 tokenize 数据，启用截断，启用填充
def preprocess_function(examples):
    return tokenizer(examples['text1'], examples['text2'], truncation=True, padding=True)

# 对数据进行预处理
train_dataset = train_dataset.map(preprocess_function, batched=True)
val_dataset = val_dataset.map(preprocess_function, batched=True)

# 加载预训练模型
model = LlamaForSequenceClassification.from_pretrained("../baichuan-LLM", num_labels=2, trust_remote_code=True)

# 定义训练参数
training_args = TrainingArguments(
    output_dir="./results",
    evaluation_strategy="epoch",
    learning_rate=2e-5,
    per_device_train_batch_size=2,
    per_device_eval_batch_size=2,
    num_train_epochs=3,
    weight_decay=0.01,
)

# 定义计算准确率的函数
def compute_metrics(eval_pred):
    logits, labels = eval_pred
    predictions = np.argmax(logits, axis=-1)
    return {'accuracy': accuracy_score(labels, predictions)}

# 创建 Trainer
trainer = Trainer(
    model=model,
    args=training_args,
    train_dataset=train_dataset,
    eval_dataset=val_dataset,
    compute_metrics=compute_metrics,
)

# 开始微调
trainer.train()

# 在验证集上进行推理
results = trainer.evaluate()
print(results)
