from transformers import AutoModelForCausalLM, AutoTokenizer, Trainer, TrainingArguments
from datasets import load_dataset, Dataset
import pandas as pd
from datasets import load_metric

# 加载数据
train_data = pd.read_csv('task2/Text_Summarization/dataset/temp-train.csv')
val_data = pd.read_csv('task2/Text_Summarization/dataset/temp-val.csv')
test_data = pd.read_csv('task2/Text_Summarization/dataset/temp-test.csv')

# 转换数据格式
train_dataset = Dataset.from_pandas(train_data)
val_dataset = Dataset.from_pandas(val_data)
test_dataset = Dataset.from_pandas(test_data)


# 加载分词器和模型，允许自定义参数
tokenizer = AutoTokenizer.from_pretrained('task2/baichuan-LLM', trust_remote_code=True)  # Use the appropriate model name
model = AutoModelForCausalLM.from_pretrained('task2/baichuan-LLM', trust_remote_code=True)

# 对数据进行分词
def tokenize_function(examples):
    inputs = examples['source']
    targets = examples['target']
    model_inputs = tokenizer(inputs, max_length=512, truncation=True)   #截断长度，允许截断
    with tokenizer.as_target_tokenizer():
        labels = tokenizer(targets, max_length=128, truncation=True)
    model_inputs["labels"] = labels["input_ids"]
    return model_inputs

# 对每个数据集进行分词
train_dataset = train_dataset.map(tokenize_function, batched=True)
val_dataset = val_dataset.map(tokenize_function, batched=True)
test_dataset = test_dataset.map(tokenize_function, batched=True)


training_args = TrainingArguments(
    output_dir='task2/Text_Summarization/model',
    evaluation_strategy="epoch",    #   每个epoch进行评估
    learning_rate=2e-5,
    per_device_train_batch_size=4,
    per_device_eval_batch_size=4,
    num_train_epochs=3,
    weight_decay=0.01,
    save_total_limit=1, # 只保留一个最新模型
)

trainer = Trainer(
    model=model,
    args=training_args,
    train_dataset=train_dataset,
    eval_dataset=val_dataset,
)

# Train the model
trainer.train()


# Load the metric
metric = load_metric('rouge')




# 生成摘要
def generate_summary(batch):
    inputs = tokenizer(batch["source"], padding="max_length", truncation=True, max_length=512, return_tensors="pt") # 返回张量的格式
    input_ids = inputs.input_ids.to(model.device)
    attention_mask = inputs.attention_mask.to(model.device)
    outputs = model.generate(input_ids, attention_mask=attention_mask, max_length=128)  # 生成摘要
    batch["pred_summary"] = tokenizer.batch_decode(outputs, skip_special_tokens=True)   #解码生成的结果，跳过特殊标记
    return batch

# 对每个测试数据生成摘要
test_dataset = test_dataset.map(generate_summary, batched=True, batch_size=4)

# 计算分数
predictions = test_dataset["pred_summary"]
references = test_dataset["target"]
rouge_result = metric.compute(predictions=predictions, references=references)


print(rouge_result)
