import json
import numpy as np,os
from PIL import Image, ImageDraw
import cv2

def image_to_json2(image_path, output_dir):
    if not os.path.exists(output_dir):
        os.makedirs(output_dir)
    image_paths=os.listdir(image_path)

    for i, path in enumerate(image_paths):
        filename = path.replace('.png', '.json')
        output_path = os.path.join(output_dir, filename)
        prediction_list = cv2.imread(os.path.join(image_path, path), cv2.IMREAD_GRAYSCALE)

        # 将预测结果转换为列表格式
        # prediction_list = predictions[i].tolist()

        objects = []
        for label in np.unique(prediction_list):
            if label == 19:
                continue  # 忽略背景
            mask = ( prediction_list  == label).astype(np.uint8)
            contours, _ = cv2.findContours(mask, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)
            for contour in contours:
                polygon = contour.squeeze().tolist()
                if len(polygon) < 3:
                    continue  # 忽略非多边形
                objects.append({
                    "label": str(label),
                    "polygon": polygon
                })

        json_data = {
            "imgHeight": prediction_list.shape[0],
            "imgWidth": prediction_list.shape[1],
            "objects": objects
        }

        with open(output_path, 'w') as json_file:
            json.dump(json_data, json_file, indent=4)

def image_to_json(image_paths, predictions, output_dir):
    if not os.path.exists(output_dir):
        os.makedirs(output_dir)

    for i, image_path in enumerate(image_paths):
        filename = os.path.basename(image_path).replace('_leftImg8bit.png', '_prediction.json')
        output_path = os.path.join(output_dir, filename)

        # 将预测结果转换为列表格式
        prediction_list = predictions[i].tolist()

        objects = []
        for label in np.unique(predictions[i]):
            if label == 19:
                continue  # 忽略背景
            mask = (predictions[i] == label).astype(np.uint8)
            contours, _ = cv2.findContours(mask, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)
            for contour in contours:
                polygon = contour.squeeze().tolist()
                if len(polygon) < 3:
                    continue  # 忽略非多边形
                objects.append({
                    "label": str(label),
                    "polygon": polygon
                })

        json_data = {
            "imgHeight": predictions[i].shape[0],
            "imgWidth": predictions[i].shape[1],
            "objects": objects
        }

        with open(output_path, 'w') as json_file:
            json.dump(json_data, json_file, indent=4)
def json_to_image():
    json_path=r'1'
    # Load the JSON data
    for file in os.listdir(json_path):
        if file.split('.')[1]!='json':
            continue
        json_file_path = os.path.join(json_path, file)
        with open(json_file_path, 'r') as files:
            label_data = json.load(files)

        # Create a blank image with the same size as the original image
        image_width = label_data['imageWidth']  # Example width, replace with actual width of your image
        image_height = label_data['imageHeight']  # Example height, replace with actual height of your image
        label_image = Image.new('L', (image_width, image_height), 0)

        # Draw the polygons
        draw = ImageDraw.Draw(label_image)
        for shape in label_data['shapes']:
            if shape['label'] == 'car':
                points = [(point[0], point[1]) for point in shape['points']]
                draw.polygon(points, fill=26)  # Assuming 26 is the label ID for cars
            elif shape['label'] == 'road':
                points = [(point[0], point[1]) for point in shape['points']]
                draw.polygon(points, fill=28)  # Assuming 26 is the label ID for cars

        # Save the label image
        print(file)
        label_image.save(os.path.join(r'2' , file.replace('.json','.png')) )

# Verify the output
# label_image.show()
# json_to_image()
image_to_json2("2","3")