# 导入库
import os
os.environ['CUDA_VISIBLE_DEVICES'] = '0'

import torch
import torch.nn as nn
import torch.optim as optim
import torch.nn.functional as F
from torch import optim
from torch.utils.data import Dataset, DataLoader, random_split
from tqdm import tqdm
import warnings
warnings.filterwarnings("ignore")
import os.path as osp
import matplotlib.pyplot as plt
from PIL import Image
import numpy as np
import albumentations as A
from albumentations.pytorch.transforms import ToTensorV2
torch.manual_seed(17)
# 自定义数据集CamVidDataset
class CityScapesDataset(torch.utils.data.Dataset):
    
    def __init__(self, images_dir, masks_dir):
        self.transform = A.Compose([
            A.Resize(224, 448),
            A.HorizontalFlip(),
            # A.RandomBrightnessContrast(),
            A.RandomSnow(),
            A.Normalize(),
            ToTensorV2(),
        ])
        self.ids = [file for file in os.listdir(images_dir) if file.endswith('.png')]
        self.ids2 = [file for file in os.listdir(masks_dir) if file.endswith('.png')]
        self.images_fps = [os.path.join(images_dir, image_id) for image_id in self.ids]
        self.masks_fps = [os.path.join(masks_dir, image_id) for image_id in self.ids2]


    def __getitem__(self, i):
        # read data
        image = np.array(Image.open(self.images_fps[i]).convert('RGB'))
        mask = np.array( Image.open(self.masks_fps[i]).convert('RGB'))
        image = self.transform(image=image ,mask=mask)

        return image['image'], image['mask'][: ,: ,0]

    def __len__(self):
        return len(self.ids)


# 设置数据集路径
x_train_dir = r"task3/dataset/leftImg8bit_trainvaltest/leftImg8bit/train"
y_train_dir = r"task3/dataset/gtFine/train"

x_valid_dir = r"task3/dataset/leftImg8bit_trainvaltest/leftImg8bit/val"
y_valid_dir = r"task3/dataset/gtFine/val"

train_dataset = CityScapesDataset(
    x_train_dir,
    y_train_dir,
)
val_dataset = CityScapesDataset(
    x_valid_dir,
    y_valid_dir,
)

train_loader = DataLoader(train_dataset, batch_size=8, shuffle=True)
val_loader = DataLoader(val_dataset, batch_size=8, shuffle=True)

for index, (img, label) in enumerate(train_loader):
    print(img.shape)
    print(label.shape)

    plt.figure(figsize=(10, 10))
    plt.subplot(221)
    plt.imshow((img[0, :, :, :].moveaxis(0, 2)))
    plt.subplot(222)
    plt.imshow(label[0, :, :])

    plt.subplot(223)
    plt.imshow((img[6, :, :, :].moveaxis(0, 2)))
    plt.subplot(224)
    plt.imshow(label[6, :, :])
    plt.show()
    if index == 0:
        break
