import json
import numpy as np
import cv2
import torch
from torch.utils.data import Dataset

# 读取JSON标注文件
with open('berlin_000000_000019_gtFine_polygons.json') as f:
    polygons = json.load(f)

# 解析标注信息，转换为掩码
height = polygons['imgHeight']
width = polygons['imgWidth']
masks = np.zeros((height, width), dtype=np.uint8)

for obj in polygons['objects']:
    label = obj['label']
    polygon = np.array(obj['polygon'], np.int32)
    cv2.fillPoly(masks, [polygon], 1)  # 1表示前景，0表示背景

# 可视化掩码
cv2.imshow('Mask', masks)
cv2.waitKey(0)
cv2.destroyAllWindows()


import torchvision
from torchvision import models
import torch.optim as optim

# 加载DeepLab v3模型
model = models.segmentation.deeplabv3_resnet101(pretrained=True)
model.classifier[4] = torch.nn.Conv2d(256, 19, kernel_size=(1, 1), stride=(1, 1))  # Cityscapes有19个类别
model = model.cuda()

# 定义损失函数和优化器
criterion = torch.nn.CrossEntropyLoss()
optimizer = optim.Adam(model.parameters(), lr=0.001)

# 训练循环
for epoch in range(epochs):
    model.train()
    for images, labels in train_loader:
        images, labels = images.cuda(), labels.cuda()
        outputs = model(images)['out']
        loss = criterion(outputs, labels)
        optimizer.zero_grad()
        loss.backward()
        optimizer.step()



import matplotlib.pyplot as plt

model.eval()
with torch.no_grad():
    for images, labels in test_loader:
        images, labels = images.cuda(), labels.cuda()
        outputs = model(images)['out']
        preds = torch.argmax(outputs, dim=1)

        # 可视化原始图像、真实标注和预测结果
        plt.figure(figsize=(10, 10))
        plt.subplot(1, 3, 1)
        plt.imshow(images.cpu().numpy()[0].transpose(1, 2, 0))
        plt.title('Original Image')
        plt.subplot(1, 3, 2)
        plt.imshow(labels.cpu().numpy()[0])
        plt.title('Ground Truth')
        plt.subplot(1, 3, 3)
        plt.imshow(preds.cpu().numpy()[0])
        plt.title('Prediction')
        plt.show()



