import os
import tensorflow as tf
from tensorflow.keras.layers import *
from tensorflow.keras.models import Model
from tensorflow.keras.optimizers import Adam
from tensorflow.keras.callbacks import ModelCheckpoint, EarlyStopping
from tensorflow.keras.preprocessing.image import load_img, img_to_array

# 数据集路径
dataset_path = 'task3/dataset/gtFine'

# 定义一些超参数
IMG_HEIGHT, IMG_WIDTH, IMG_CHANNELS = 256, 256, 3
NUM_CLASSES = 34
BATCH_SIZE = 8
EPOCHS = 50
LEARNING_RATE = 1e-4

# 数据生成器
def data_generator(dataset_path, batch_size, img_height, img_width):
    # 实现数据加载和预处理的生成器函数
    pass  # 请在此处实现数据生成器

# 创建DeepLab v3模型
def create_deeplabv3(input_shape, num_classes):
    base_model = tf.keras.applications.Xception(weights='imagenet', include_top=False, input_shape=input_shape)
    x = base_model.output
    x = Conv2D(256, (3, 3), padding='same', dilation_rate=(6, 6))(x)
    x = BatchNormalization()(x)
    x = Activation('relu')(x)
    
    x = Conv2D(256, (3, 3), padding='same', dilation_rate=(12, 12))(x)
    x = BatchNormalization()(x)
    x = Activation('relu')(x)
    
    x = Conv2D(256, (3, 3), padding='same', dilation_rate=(18, 18))(x)
    x = BatchNormalization()(x)
    x = Activation('relu')(x)
    
    x = Conv2D(256, (3, 3), padding='same', dilation_rate=(24, 24))(x)
    x = BatchNormalization()(x)
    x = Activation('relu')(x)
    
    x = Conv2D(num_classes, (1, 1), padding='same')(x)
    x = tf.image.resize(x, (input_shape[0], input_shape[1]))
    x = Activation('softmax')(x)
    
    model = Model(inputs=base_model.input, outputs=x)
    
    return model

# 创建模型
model = create_deeplabv3((IMG_HEIGHT, IMG_WIDTH, IMG_CHANNELS), NUM_CLASSES)
model.compile(optimizer=Adam(learning_rate=LEARNING_RATE), loss='sparse_categorical_crossentropy', metrics=['accuracy'])

# 模型保存与早停
checkpoint = ModelCheckpoint('deeplabv3_cityscapes.h5', monitor='val_loss', save_best_only=True)
early_stopping = EarlyStopping(monitor='val_loss', patience=10)

# 数据生成器实例
train_generator = data_generator(os.path.join(dataset_path, 'train'), BATCH_SIZE, IMG_HEIGHT, IMG_WIDTH)
val_generator = data_generator(os.path.join(dataset_path, 'val'), BATCH_SIZE, IMG_HEIGHT, IMG_WIDTH)

# 训练模型
model.fit(train_generator, validation_data=val_generator, epochs=EPOCHS, callbacks=[checkpoint, early_stopping])

# 保存模型
model.save('deeplabv3_cityscapes_final.h5')

import matplotlib.pyplot as plt

def visualize_segmentation_results(model, dataset_path, num_samples=5):
    test_generator = data_generator(os.path.join(dataset_path, 'test'), 1, IMG_HEIGHT, IMG_WIDTH)
    
    for i in range(num_samples):
        img, true_mask = next(test_generator)
        pred_mask = model.predict(img)
        
        img = img[0]
        true_mask = true_mask[0]
        pred_mask = tf.argmax(pred_mask, axis=-1)[0]

        plt.figure(figsize=(10, 5))
        plt.subplot(1, 3, 1)
        plt.title('Original Image')
        plt.imshow(img)
        
        plt.subplot(1, 3, 2)
        plt.title('True Mask')
        plt.imshow(true_mask)
        
        plt.subplot(1, 3, 3)
        plt.title('Predicted Mask')
        plt.imshow(pred_mask)
        
        plt.show()

# 可视化分割结果
visualize_segmentation_results(model, dataset_path)
